# 
# Indentation finder, by Philippe Fremy <phil at freehackers dot org>
# Copyright 2002,2005 Philippe Fremy
#
# This program is distributed under the BSD license. You should have received
# a copy of the file LICENSE.txt along with this software.
#
import os, glob
from indent_finder import *

def check_indentation(  indentation,
					    path_list, 	
						glob_list=['*'], 
						recursive=False ):
	'''Check a set of files for a given indentation and returns the files that do not
	match the specified indentation.

	Arguments:
	* indentation: target indentation that the files should have. One of INDENT_SPACE_? or
	               INDENT_TAB
	* path_list: a list of files or directories in which to search
	* glob_list: a list of globbing pattern to use for recursive directory searching for
	files that should be included. Defaults to * (meaning all files).
	* recursive: when one of the elements of path_list is a directory, scan the subdirectories of
	this path if recursive is true.

	Returns: a list of all the files that do not match the indentation specified in @p indentation.
	'''
	file_list = build_file_list( path_list, glob_list, recursive )
	bad_indentation = find_bad_indentation( file_list, indentation )
	return bad_indentation

def build_file_list( path_list, glob_list, recursive ):
	'''Return a list of files according to the arguments.

	See check_indentation() for the signification of the arguments.
	'''
	file_list = [ f for f in path_list if os.path.isfile( f ) ]
	dir_list = [ d for d in path_list if os.path.isdir( d ) ]
	for d in dir_list:
		expand_file_list( d, glob_list, recursive, file_list )
	file_list = [ os.path.normpath( f ) for f in file_list ]
	return file_list

def expand_file_list( dir, glob_list, recursive, file_list ):
	'''Extend the file_list with the files of the directory dir.

	The function may call itself recursively if recursive is true.
	'''
	path_list = []
	for glob_pattern in glob_list:
		path_list.extend( glob.glob( os.path.join( dir, glob_pattern ) ) )
	local_file_list = [ f for f in path_list if os.path.isfile( f ) ]
	file_list.extend( local_file_list )
	if recursive:
		dir_list = [ d for d in os.listdir( dir ) if os.path.isdir( d ) ]
		for d in dir_list:
			expand_file_list( d, glob_list, recursive, file_list )
	

def find_bad_indentation( file_list, indentation ):
	'''Return a list of (file that does not match indentation, indentation of the bad file).
	'''
	bad_indentation = []
	indentFinder = IndentFinder()
	for f in file_list:
		indentFinder.clear()
		indentFinder.parse_file( f )
		f_indentation = str(indentFinder)
		if f_indentation != indentation:
			bad_indentation.append( (f, f_indentation) )
	return bad_indentation

def main():
	usage = '''%s "indentation" file1 file2 ...'''
	if len(sys.argv) < 2: 
		print usage % sys.argv[0]
		sys.exit( -1 )

	indentation = sys.argv[1]
	indent_list = [ INDENT_TAB, INDENT_SPACE_2  , INDENT_SPACE_3  , INDENT_SPACE_4 , INDENT_SPACE_5, INDENT_SPACE_6, INDENT_SPACE_7, INDENT_SPACE_8 ]
	if not indentation in indent_list: 
		print 'Second argument must be one of ', indent_list
		sys.exit( -1 )

	bad_indentation = find_bad_indentation( sys.argv[2:], indentation )
	if len(bad_indentation) == 0:
		print 'Indentation is "%s" for all files.' % indentation
		sys.exit( 0 )

	print 'Target indentation is:', indentation
	for f in bad_indentation:
		print 'Inconsistent indentation for %s: %s' % f
	sys.exit( len(bad_indentation ) )	

if __name__ == '__main__': main()	
